(*
 * Coefficients to use for linear expression 
 *
 *)

open Gmp
type t = Q.t


val to_string: t -> string
(* parse *)

val lex_tokens: string list
val parse_coeff: Genlex.token Stream.t -> t
val of_string: string -> t
(** parsing routines *)
val to_int: t -> int
val of_int: int -> t
val of_int_pair: int -> int -> t 

(* val of_char: int -> t *)
(* val of_int64: int -> t *)
 val of_float: float -> t 
(** Conversions*)



val zero:t (** 0 *)
val one: t (** 1 *)
val minus_one: t (** -1 *)
val mone: t
(** synonym for -1 *)

val compare: t->t -> int
val is_zero: t -> bool 
val geq: t -> t-> bool
val leq: t-> t-> bool 
val eq: t-> t-> bool
val neq: t -> t -> bool
val ge: t-> t-> bool
val le: t-> t-> bool
(** comparisons etc. *)


(* 
 * Generate information
 *)

val is_int: t -> bool 
val num: t-> int
val den: t-> int

val add: t-> t-> t
val subtract: t-> t-> t
val multiply: t-> t-> t
 val divide: t-> t-> t 


val max: t -> t -> t
val min: t -> t -> t

val negate: t-> t

val add_int: int -> t -> t
val multiply_int: int -> t-> t



(** Arith operations  *)
val is_well_behaved: t -> bool
  (** use to heuristically judge if the coefficients are large and 
     the algorithm is in the throes of numerical instability? *)


