module S=String
module L=List



module Strmap = Map.Make (
   struct 
     type t = string
     let compare s1 s2 = compare s1 s2 
   end )

module Intmap = Map.Make (
   struct 
     type t = int
     let compare = (-)
   end )

type 'a intmap = 'a Intmap.t

type 'a smap = 'a Strmap.t


module Strset = Set.Make (
  struct 
    type t=string
    let compare= S.compare
  end
)

type sset=Strset.t


module Intset = Set.Make (
  struct 
    type t=int
    let compare= (-)
  end
)

type intset=Intset.t
(* 
let  gcd i j = 
  let bi=big_int_of_int i in 
   let bj= big_int_of_int j in 
   let bg = gcd_big_int bi bj in
   int_of_big_int bg
*)


module Strhash = Hashtbl.Make (
  struct
    type t = string
    let equal s1 s2 = (S.compare s1 s2) == 0
    let hash = Hashtbl.hash
  end
)   
   

module Inthash = Hashtbl.Make (
  struct
    type t = int
    let equal = (==)
    let hash = Hashtbl.hash
  end
)




exception Violation

let check_ascending_sorted l comp = 
  let n =List.length l in
      try
	begin
	  for i = 0 to n-2 do
	    let (a1,a2) = (L.nth l i), (L.nth l (i+1)) in
	      if (comp a1 a2) >= 0 then raise Violation
	  done;
	  true
	end
      with 
	| Violation -> false
	  

let rec strip_some l = match l with (* strip Some/None elements out of a list *)
  | [] -> []
  | None:: l -> strip_some l
  | (Some x) :: l ->  x:: (strip_some l)
      


let weave fn lst lst1 =
  (* apply fn to each pair i,j with i in lst and j in lst1 *)
  match (lst,lst1) with
    | [],_ -> []
    | _, [] -> []
    | _ -> 
	try
	  let m = L.length lst in 
	  let n = L.length lst1 in 
	  let retlst = ref [] in
	    for i = 0 to n-1 do
	      for j = 0 to m-1 do
		let e = L.nth lst1 i in
		let e1= L.nth lst j in
		  retlst := (fn e1 e)::!retlst
	      done
	    done;
	    !retlst
	with 
	  | Failure s -> 
	      raise (Failure (s^"Glbdefs.weave"))




let weave_iter fn lst lst1 =
  (* apply fn to each pair i,j with i in lst and j in lst1 *)
  match (lst,lst1) with
    | [],_ 
    | _, [] -> ()
    | _ -> 
	try
	  let m = L.length lst in 
	  let n = L.length lst1 in 
	    for i = 0 to n-1 do
	      let e = L.nth lst1 i in
		for j = 0 to m-1 do  
		  let e1= L.nth lst j in
		    fn e1 e
		done
	    done
	with 
	  | Failure s -> 
	      raise (Failure (s^"Glbdefs.weave"))




let sset_to_string s ss = 
  let retstr = ref "" in
  let han s1 =   retstr := !retstr^s^s1 
  in
    Strset.iter han ss;
    !retstr

let lst_to_string pfun lst = 
  let retstr = ref "" in 
  let hand a = 
    retstr := !retstr^"  "^(pfun a) 
  in
    L.iter hand lst;
    !retstr



let get_memory_used ()  =
   try 
   let fd = Unix.openfile "/proc/self/statm" ([Unix.O_RDONLY]) 0o640 in
   let icc = Unix.in_channel_of_descr fd in 
   let mem = ref (-1) in 
   let get_mem i = mem := i in
   let ()= Scanf.fscanf icc "%d" get_mem in
   let () = Unix.close fd in
          4* (!mem)
   with
     | _ -> Printf.printf " Error reading ppack resource usage.\n"; 
                (-1)



type context = int -> string

let getName ctx i = 
  ctx i



let varList = ref Strmap.empty
let nVars = ref 0
let strList = ref []

let addVar str = 
  (varList := Strmap.add str !nVars !varList);
  strList := !strList@([str]);
  incr nVars

let getID str = 
  try
    Strmap.find  str !varList
  with 
    | Not_found -> failwith (Printf.sprintf "Cannot find string %s \n" str)


let varToString i = 
try
  List.nth !strList i
with 
  | _ -> Printf.sprintf "x%d" i

  

let numVars () = !nVars



  

let getIntegerSet i j = 
  let vSet = ref Intset.empty in 
  for k = i to (j-1)
  do
    vSet := Intset.add k !vSet
  done;
  !vSet


let iterIndexed foo lst = 
  let i = ref 0 in 
  let hElt l = 
    foo !i l;
    incr i
  in 
    List.iter hElt lst

