{
  open Sys_parser
  let linenum = ref 1
}

let whitespace = [' ' '\t']

rule lexer = parse
  | whitespace {lexer lexbuf}
  | '\n' {incr linenum; lexer lexbuf }
  | "#" [^ '\n']* { lexer lexbuf }
  | "%" [^ '\n']* {lexer lexbuf }
  | "dynamics" { LDer }
  | "dyn" { LDer }
  | "der" { LDer }
  | "system" { LSystem }
  | "mode" { LMode }
  | "System" { LSystem}
  | "Mode" { LMode}
  | "<-" { LArrow}
  | "{" { LOpenCurly }
  | "}" { LCloseCurly }
  | "(" { LOpenBrace }
  | ")" { LCloseBrace }
  | ";" { LSemiColon }
  | "," { LComma }
  | "+" { LPlus }
  | "-" { LMinus}
  | "*" { LMult }
  | "^" { LPower} 
  | "/" { LDiv}
  | "[]" { LBar }
  | "->" { LFwdArrow }  
  | ['0'-'9']* '.' ['0'-'9']+ ('E' ('-')? ['0'-'9']+)? { LFlt (Lexing.lexeme lexbuf) }
  |  ['0'-'9']+  { LInt (int_of_string (Lexing.lexeme lexbuf))}
  |  ['A'-'Z' 'a'-'z']['0'-'9' '_' 'A'-'Z' 'a'-'z']*  { LIdent (Lexing.lexeme lexbuf) }
  | _ as  x { failwith (Printf.sprintf "Error %c line %d " x !linenum) }

