%{
  open Glbdefs
  open Mpoly

  let deglim = ref 3
%}

%token <int> LInt
%token <string> LIdent
%token <string> LFlt
%token LSystem LMode LArrow 
%token LOpenCurly LCloseCurly LOpenBrace LCloseBrace
%token LSemiColon LComma LPlus LMinus LMult
%token LDiv LPower
%token LDecimal LFlt LBar LFwdArrow LDer 

%left LPlus LMinus
%left LMult LDiv
%right LPower
%right UNARYMINUS

%start system
%type<unit> system

%%

system: LSystem LOpenBrace nameList LCloseBrace LOpenCurly system_modes system_transitions LCloseCurly {}
;

nameList: LIdent { Glbdefs.addVar $1}
| nameList LComma LIdent { Glbdefs.addVar $3}
;

system_modes:  {}
| mode_descr system_modes { }
;


mode_descr: LMode LIdent LOpenCurly LDer LOpenCurly dynamics_descr LCloseCurly   LCloseCurly {
  System.addMode $2 $6
}
| LMode LIdent LOpenCurly dynamics_descr LCloseCurly {
    System.addMode $2 $4 
  }
;

system_transitions: {}
| trs_descr system_transitions {}
;

trs_descr: LBar LIdent LFwdArrow LIdent LOpenCurly dynamics_descr LCloseCurly {
   let preID = 
     try 
       System.findModeIDByName  $2 
     with | Not_found -> failwith ("Mode: "^$2^" not found. ") 
   in 
   let postID = 
     try 
       System.findModeIDByName $4 
     with | Not_found -> failwith ("Mode: "^$4^" not found. ") 
   in 
     System.addTransition preID postID $6

}

;

dynamics_descr: {
  Intmap.empty
}
| identifier LArrow mpoly dynamics_descr {
    let p = MP.zero (Glbdefs.numVars () )  
    in 
      List.iter (fun m -> MP.addTermAssign m p) $3;
      Printf.printf "%s <- %s \n" (Glbdefs.varToString $1) 
	(MP.to_string p);
      Intmap.add $1 p $4
}
;


mpoly:  mpoly LPlus mterm {
 match $3 with 
   | c , (None) -> 
       ((c), (MP.zero_monomial (Glbdefs.numVars ()) ))::$1
   | c, (Some m) -> 
       (c,m)::$1

}
| mpoly LMinus mterm {
    match $3 with 
      | c , (None) -> 
	  ( (Coeff.negate c), (MP.zero_monomial (Glbdefs.numVars ()) ) ) ::$1
      | c, (Some m) -> 
	  ((Coeff.negate c),m)::$1
}
| mterm {
    match $1 with 
      | c , (None) -> 
	  [(c), (MP.zero_monomial (Glbdefs.numVars ()) )]
      | c, (Some m) -> 
	  [(c,m)]

  }
| LMinus mterm %prec UNARYMINUS {
    match $2 with 
      | c , (None) -> 
	  [(Coeff.negate c), (MP.zero_monomial (Glbdefs.numVars ()) )]
      | c, (Some m) -> 
	  [((Coeff.negate c),m)]
	  
} 
;

mterm: coeff LMult monomial {
  $1, (Some $3)
}
| monomial {
  (Coeff.one), (Some $1)
}
| coeff {
    ($1, None)
  }
;


monomial: identifier {
  let m1 = MP.zero_monomial (Glbdefs.numVars ()) in 
    MP.add_power (m1 ) $1 1;
    m1
}
| identifier LPower LInt {
    let m1 = MP.zero_monomial (Glbdefs.numVars ()) in 
      MP.add_power (m1 ) $1 $3;
      m1
}
| monomial LMult identifier {
    MP.add_power $1 $3 1;
    $1
}
| monomial LMult identifier LPower LInt {
    MP.add_power $1 $3 $5;
    $1
  }
;

identifier: LIdent {
    Glbdefs.getID $1
 
}
;

coeff: LInt LDiv LInt {Coeff.of_int_pair $1 $3 }
| LInt LDecimal LInt { 
    let str = Printf.sprintf "%d.%d" $1 $3 in 
      Coeff.of_float (float_of_string str) 
  }
| LFlt  {
      Coeff.of_float (float_of_string $1) 
  }
| LInt {
    Coeff.of_int $1
  }
;


%%
