
open Glbdefs

(* This is the module for multivariate polynomials *)
open Gmp

module MP = Mpoly.MP


(* We now create a module for template polynomials *)

module ParamExprs = struct
  type t = Linexp.t
  let zero = Linexp.const_expr Coeff.zero
  let one = Linexp.const_expr Coeff.one
  let mone = Linexp.const_expr Coeff.mone
  let minus_one = Linexp.const_expr Coeff.minus_one
  let add = Linexp.add_linexp
  let divide n1 n2 = assert false
  let subtract = Linexp.subtract_linexp
  let negate = Linexp.multiply_linexp (Coeff.minus_one) 
  let compare = Linexp.compare
  let eq a b= (Linexp.compare a b) = 0
  let geq a b = (Linexp.compare a b) >= 0
  let leq a b = (Linexp.compare a b) <= 0
  let le a b = (Linexp.compare a b) < 0
  let ge a b = (Linexp.compare a b) > 0
  let neq a b = (Linexp.compare a b ) < 0
  let pVar i = Printf.sprintf "C(%d)" i
  let to_string a = "["^(Linexp.to_string pVar a)^"]"

  let of_int  i = Linexp.const_expr (Coeff.of_int i)
  let is_zero = Linexp.is_zero
  let multiply a b = 
    if (Linexp.is_constant a) 
    then (
      Linexp.multiply_linexp (Linexp.get_constant a) b
    ) else  (
      if (Linexp.is_constant b) then (
	Linexp.multiply_linexp (Linexp.get_constant b) a
      ) else (
	assert false (* multiplication of template expressions is not supported. therefore we should not try to do this *)
      )
    )
  let of_string str = zero (*-- disabled for now --*)
end



module TP = struct
  include Mpoly.S(ParamExprs)

    
(* 
 *  We need to support operations to 
 *
 *  1. create templates as generic templates of a given degree, 
 *  2. operations that involve templates and 
 *  3. regular ones like taking lie derivatives. 
 *)
 
  let ofMPoly (mp: MP.t) = 
    let n = mp.MP._n in 
    let pRet = zero n in 
    let hTerm ((c:Coeff.t),(m:mon)) = 
      let lc = Linexp.const_expr c in 
	addTermAssign (lc,m) pRet
    in
    MP.iterForEachTerm hTerm mp;
    pRet
      
      
  let scaleMPTerm ((c:Coeff.t), (m:mon)) (p:t) = 
    assert( not (Coeff.is_zero c));
    let hTerm (m1:mon) (l:Linexp.t)  (nMap:Linexp.t MMap.t) = 
      let m2 = monomial_product m m1 in 
      let l' = Linexp.multiply_linexp c l in 
      if not (Linexp.is_zero l') 
      then  MMap.add m2 l' nMap
      else assert false 
    in
    let pRet = copy p in 
    let nMap = MMap.fold hTerm pRet._m MMap.empty in
    pRet._m <- nMap;
    pRet
      
      
      
  let product (tp:t) (mp:MP.t) = 
    (*-- 
      product of a multivariate polynomial and a template polynomial.
      --*)
    assert( tp._n = mp.MP._n);
    let tpRet = zero tp._n in 
    MP.iterForEachTerm ( fun (c,m) -> ( addAssign tpRet (scaleMPTerm (c,m) tp) )) mp;
    tpRet

  let lieDerivative (vField:MP.t list) (p:t) =
    assert ((List.length vField) = (p._n));
    let retP = zero p._n in 
    let gr = grad p in 
    let hRes pJ  mpJ= 
      addAssign retP (product pJ mpJ) 
    in
    List.iter2 hRes gr vField;
    retP

      
      
  let fPreMonomial (trs: (MP.t) list) (c, (m: mon)) =
    let n = List.length trs in 
    let termP = ref (const  n c) in   (* Other wise , compute pre of the monomial *)
      for i = 0 to (n -1) 
      do
	let dI =  m.(i) in 
	let trsI = try List.nth trs i with Failure _ -> assert false in 
	  for j = 0 to (dI -1)
	  do
	    termP := product  !termP trsI 
	  done;   
      done;
      !termP

  let fPre (trs: (MP.t) list) (p:t) = 
    assert( (List.length trs) = p._n); (* The update better have as many vars as the polynomial *)
    let retP = zero p._n in  (* What I plan to return *)
    let hTerm (c,m) =  (* Iterate through the terms of the polynomial *)
      if (is_zero_monomial m)  (* If it is the constant term *)
      then addTermAssign (c,m) retP (* simply add it to the return *)
      else (
	let termP = fPreMonomial trs (c,m) in 
	addAssign retP termP
      )
    in 
      iterForEachTerm hTerm p;
      retP
      
  let makeTP n mList = 
    let retTP = zero n in
    let nTemplVars = ref 1 in
      addTermAssign ( ( Linexp.expr_of Coeff.one 0), (zero_monomial n) ) retTP;
      if (List.length mList ) > 0
      then (
	let hTerm ( (c:Coeff.t), (m:mon) ) = 
	  let l = Linexp.expr_of c !nTemplVars in 
	    addTermAssign (l,m) retTP 
	in 
	let handlePolynomial (p:MP.t) = 
	  MP.iterForEachTerm hTerm p ;
	  incr nTemplVars
	in 
	  List.iter handlePolynomial mList ;
      );
      (!nTemplVars), retTP
      
      
  let createTemplate nVars degLimit = 
    let nTemplVars = ref 0 in
    let retTP = zero nVars in 
    let addOneTerm lst = 
      assert(List.length lst = nVars);
      let m = Array.of_list lst in (*- make a monomial -*)
      let l = Linexp.expr_of Coeff.one !nTemplVars in (*- make a linear expression -*)
	incr nTemplVars; (*- increment the count of variables -*)
	addTermAssign (l,m) retTP
    in 
    let rec createMon degSoFar lst = 
      assert (degSoFar >= 0 );
      assert (degSoFar <= degLimit);
      assert ((List.length lst ) <= nVars);
      if (List.length lst = nVars) 
      then (*-- we are done, make a monomial and create a new template variable --*)
	addOneTerm lst 
      else
	(
	  if (degSoFar = 0) 
	  then createMon degSoFar (0::lst)
	  else (
	    for i = 0 to (degSoFar)
	    do
	      createMon (degSoFar - i) (i::lst)
	    done
	  )
	)
    in 
      createMon degLimit [] ;
     (!nTemplVars, retTP)


  let normalizeTP (tp:t) (eq: Eqs.t) = 
    let tpRet = zero tp._n in 
    let hTerm ((l:Linexp.t),(mo:mon)) = 
      let l' = (Eqs.normal_form eq l) in 
	if (Linexp.is_zero l')
	then ()
	else 
	  addTermAssign (l',mo) tpRet
    in 
      iterForEachTerm hTerm tp ;
      tpRet


  let instantiateParams (tp:t) (lSol:Linexp.t) = 
    let mpRet = MP.zero tp._n in 
    let hTerm  ((l:Linexp.t),(mo:mon)) = 
      let (c,d) = Linexp.dot_product lSol l in 
	MP.addTermAssign ((Coeff.add c d), mo) mpRet 
    in 
      iterForEachTerm hTerm tp;
      mpRet

  let collectOverflowTerms tpDeriv tp = 
    let retList = ref [] in 
    let tpRet = zero tp._n in 
    let hTerm ((l:Linexp.t), (mo:mon) )  = 
      (*-- check if the term has a non-zero coeff in tp ---*)
      let l1 = get_coefficient tp mo in 
	if (Linexp.is_zero l1)
	then (
	    retList := l::!retList (* no, add coefficient to list, we have to zero it out next *)
	)
	else addTermAssign (l1,mo) tpRet  (* yes, great, we will keep it. *)
    in 
      iterForEachTerm hTerm tpDeriv;
      (!retList),tp


  let encodeMembership nTemplVars eq pDeriv tp = 
    let hTerm ((l: Linexp.t), (mo: mon)) = 
      let c1 = MP.get_coefficient pDeriv mo in 
      let l1 = Linexp.add_constant (Coeff.negate c1) l in 
	Eqs.add_equation eq l1
    in 
      iterForEachTerm hTerm tp

  let matchTerms nTemplVars eq tpDeriv tp = 
    let transform l1 = Linexp.transform (fun s c -> ( (s+nTemplVars), c ) ) l1 in 
    let hTerm ((l:Linexp.t), (mo:mon) )  = 
      (*-- check if the term has a non-zero coeff in tp ---*)
      (  
	let l1 = get_coefficient tp mo in 
	  if (Linexp.is_zero l1) 
 	  then ( 
  	    Eqs.add_equation eq l
	  ) else 
	    ( (*-- add nTemplVars to the coefficients of l1 --*)
	      let l2 = transform l1 in 
		Eqs.add_equation eq (Linexp.subtract_linexp l l2) (*- add l - l2 = 0 -*)
	    )
      )
    in 
    let hTermTP ((l:Linexp.t), (mo:mon) ) = 
      let l1 = get_coefficient tpDeriv mo in 
	if (Linexp.is_zero l1) 
	then Eqs.add_equation eq (transform l)
    in 
      iterForEachTerm hTerm tpDeriv;
      iterForEachTerm hTermTP tp

	 
	 
end
